/*
  This file is part of ZitherLayout.

  ZitherLayout is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  ZitherLayout is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ZitherLayout.  If not, see <http://www.gnu.org/licenses/>.


  Diese Datei ist Teil von ZitherLayout.

  ZitherLayout ist Freie Software: Sie können es unter den Bedingungen
  der GNU General Public License, wie von der Free Software Foundation,
  Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren
  veröffentlichten Version, weiterverbreiten und/oder modifizieren.

  ZitherLayout wird in der Hoffnung, dass es nützlich sein wird, aber
  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
  Siehe die GNU General Public License für weitere Details.

  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
*/

/*
**  © 2011-2012 Carsten Gaede (PiperSoft)
**
**  @project: ZitherLayout
**  @file: definitions.h
**  @brief definition of default values and structures
**
**  @date: 2011-11-13
**  @author: Carsten Gaede
**
**
**  @version 1.0.0.0 2012-01-01 Carsten Gaede
**    first official release
**  @version 0.0.0.1 2011-11-13 Carsten Gaede
**    initial creation
*/


#ifndef DEFINITIONS_H_
  #define DEFINITIONS_H_
	#define DEFINITIONS_H__ACTIVE


  //*** definitions ************************************************************

  // maximum length of input line
  #define MAX_INPUT_LENGTH  1024

  // maximum number of notes
  #define MAX_NOTES 256
  // maximum length of info fields
  #define MAXLEN_TITLE 64
  #define MAXLEN_SUBTITLE 256
  #define MAXLEN_ARTIST 32
  #define MAXLEN_ARRANGEMENT MAXLEN_ARTIST
  #define MAXLEN_WARNING  128
  #define MAXLEN_HINT 128
  #define MAXLEN_LYRICS MAX_INPUT_LENGTH
  // maximum number of lyrics
  #define MAX_LYRICS  16

  // maximum pitch
  #define DEF_MAX_PITCH 25  // default max pitch, may be changed

  // layout styles
  #define LAYOUT_STYLE_MELODY 0
  #define LAYOUT_STYLE_BASS   1


  // definition of length values
  #define LENGTH_NONE     0x00
  #define LENGTH_INVALID  0xFF
  #define LENGTH_1        0x01
  #define LENGTH_2        0x02
  #define LENGTH_4        0x03
  #define LENGTH_8        0x04
  #define LENGTH_16       0x05
  #define LENGTH_32       0x06
  #define LENGTH_64       0x07
  //#define LENGTH_128      0x08
  //#define LENGTH_256      0x09
  #define LENGTH_MASK     0x3F
  #define LENGTH_PT       0x80  // pointed note
  //#define LENGTH_DPT      0x40  // double pointed note

  // definition of values
  #define VALUE_NONE    0
  #define VALUE_PAUSE   0xFF

  // definition of chords
  #define CHORD_VALUE_MASK  0x0F
  #define CHORD_PT          0x10  // pointed chord
  #define CHORD_SLR         0x20  // slured chord
  #define DEF_MAX_CHORD     6   // default value, may be changed

  // definition of relative positions
  #define RPOS_NONE   0
  #define RPOS_LEFT   1
  #define RPOS_TL     2
  #define RPOS_TOP    3
  #define RPOS_TR     4
  #define RPOS_RIGHT  5
  #define RPOS_BR     6
  #define RPOS_BOTTOM 7
  #define RPOS_BL     8
  #define RPOS_L  RPOS_LEFT
  #define RPOS_R  RPOS_RIGHT
  #define RPOS_T  RPOS_TOP
  #define RPOS_B  RPOS_BOTTOM


  #define MAX_INPUT_FILES 16  // maximum number of input files
  #define MAX_CL_SETTINGS 16  // maximum number of command line settings items

  // definition of log modes
  #define LOG_MODE_QUIET    0
  #define LOG_MODE_NORMAL   1
  #define LOG_MODE_VERBOSE  2

  //*** structure definitions **************************************************

  struct POSITION {
    unsigned char row, col;   // logical position
    short x, y;               // absolute position
  };

  //*** settings structure definition ***
  // definition of dimensions preset
  #define NORMAL_DIMENSIONS 0
  #define SMALL_DIMENSIONS  1
  #define TINY_DIMENSIONS   2

  // notes positions
  #define NOTES_POS_TOP     0
  #define NOTES_POS_CENTER  1
  #define NOTES_POS_BOTTOM  2


  struct INPUT_DEFAULTS {
    unsigned char length;        // default length if omitted, set with: \dl
    unsigned char octave;        // default octave if omotted, set with: \do
    unsigned char octave_start;
  };
  // dimensions in 1/10 mm
  struct DIMENSIONS {
    struct {
      unsigned short height, width;                       // set with: \ph / \pw
      struct {
        unsigned short top, bottom, left, right;          // set with: \pb / \pbt / \pbb / \pbl / \pbr   (page border)
      } borders;
    } page;
    struct {
      unsigned short height, width;                       // calculated with page dimensions and page borders
      struct {
        unsigned short width;                             // calculated
        unsigned short offset;                            // calculated (if note part aligned right hand (melody style))
        struct {
          unsigned short top, bottom, left, right;        // calculated, set with: \nbl / \nbr (note border left / right)
        } borders;
      } notes;
      struct {
        unsigned short width;                             // calculated
        unsigned short offset;                            // calculated (if info part aligned right hand (bass style))
        unsigned short c1_dist;                           // set with \nbr
        struct {
          unsigned short top, bottom;                     // calculated (set to notes borders)
        } borders;
        struct {
          unsigned short border;
          unsigned short width;
          unsigned short sub_title_dist;
        } title;
      } info;
      struct {
        unsigned short height;                            // fix, depending on note size (which depends on max row needed)
                                                          // may be changed later to fit all notes on the page
      } row;
      struct {
        unsigned short width;                             // set with: \sd (string distance)
      } col;
    } picture;
    struct {
      unsigned short width;                               // fix, depending on note size
      unsigned short width_dotted;                        // fix, depending on note size
      unsigned short start_needle_size;                   // calculated with note width
      unsigned short circle_size;                         // calculated with note width
      short dx;                                           // calculated with note width
      short dy;                                           // fix, always 0.9mm
      struct {
        unsigned short dist;
        unsigned short width;
        unsigned short height;
        short dx_l;                                       // calculated with width and note width
        short dx_r;                                       // calculated with width and note width dotted
        short dx_tb;                                      // calculated with width
        short dy_lr;                                      // calculated with height
        short dy_b;                                       // calculated with height and note height
      } chord;
    } notes;
    struct {
      unsigned short number_dist;
    } lyrics;
    struct {
      unsigned short height, width;                       // set with \ceh/\cew, (cut edge height / width) default available
    } cut;
    struct {
      unsigned short width;                               // set with \cw (chord width)
      unsigned short c1_dist;                             // set with \cd (chord (to c1) distance)
      unsigned short dist;                                // set with \sd
    } chords;
  };
  struct COUNTERS {
    unsigned char cols;                                   // set with: \sc (string count)
    unsigned char rows;                                   // calculated with pic.h, nbt,nbb and height
    unsigned char chords;                                 // set with: \cc (chord count)
  };
  struct SETTINGS {
    unsigned char layout_style;                           // set with \s
    struct {
      char *items[MAX_CL_SETTINGS];
      unsigned char count;
    } cmd_line;
    struct INPUT_DEFAULTS defaults;
    unsigned char grid;   // print grid?                  // set with \g
    unsigned char lyric_numbers;                          // set with \ln
    struct DIMENSIONS dim;
    struct COUNTERS count;
    unsigned char notes_position;                         // set with: \np (center is default)
    struct {
      unsigned char min;                                  // set with: \lc/\fc (lowest compression), default 0
      unsigned char max;                                  // set with: \hc/\fc (highest compression), default depending on lc and some dimensions
      unsigned char current;
    } compression;
  };


  //*** song structure definition ***
  struct NOTE {
    unsigned char length; // note length and point flag
    unsigned char value;  // x position (=col)
    unsigned char chord;  // chord value and type flag
    unsigned char dir;    // up, down, turn, stay
    struct POSITION pos;  // position
    unsigned char line_pos_in;  // relative position of line coming from previous note
    unsigned char chord_pos;    // relative position of chord

  };
  struct NOTES {
    struct NOTE notes[MAX_NOTES];
    unsigned short count;
  };
  struct LYRIC {
    char text[MAXLEN_LYRICS + 1];             // set with @l<x>
    struct POSITION pos;                      // set with \lp<x>, real coordinates calculated
  };
  struct LYRICS {
    struct LYRIC lyrics[MAX_LYRICS];
    unsigned short count;
  };
  struct SONG {
    struct NOTES notes;
    char title[MAXLEN_TITLE + 1];             // set with @t(itle)
    char subtitle[MAXLEN_SUBTITLE + 1];       // set with @s(ubtitle)
    struct {
      char composer[MAXLEN_ARTIST + 1];         // set with @a(rtist) or @c(omposer)
      char texter[MAXLEN_ARTIST + 1];           // set with @a(rtist) or @te(xter)
      char arrangement[MAXLEN_ARRANGEMENT + 1]; // set with @arr(angement)
    } artists;
    char warning[MAXLEN_WARNING + 1];         // set with @w(arning)
    char hint[MAXLEN_HINT + 1];               // set with @h(int)
    struct LYRICS lyrics;
    unsigned char rows_needed;                // calculated from notes
    struct POSITION start_needle_pos;         // calculated from first note position
  };


  //*** session structure ***

  struct SESSION {
    struct {
      char *lpFileNames[MAX_INPUT_FILES];
      unsigned char uFileCount;
      unsigned char bReadStdIn;
    } input;
    struct {
      char *lpFileName;
    } output;
    unsigned char uLogMode;
    struct SETTINGS set;
    struct SONG song;
  };


  //*** generic macros ***

  #define MAX(a,b)  ((a)>(b)?(a):(b))
  #define MIN(a,b)  ((a)<(b)?(a):(b))


	#undef DEFINITIONS_H__ACTIVE
#else
	#ifdef DEFINITIONS_H__ACTIVE
		#warning circular inclusion of file 'definitions.h'
	#endif
#endif
