/*
  This file is part of ZitherLayout.

  ZitherLayout is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  ZitherLayout is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ZitherLayout.  If not, see <http://www.gnu.org/licenses/>.


  Diese Datei ist Teil von ZitherLayout.

  ZitherLayout ist Freie Software: Sie können es unter den Bedingungen
  der GNU General Public License, wie von der Free Software Foundation,
  Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren
  veröffentlichten Version, weiterverbreiten und/oder modifizieren.

  ZitherLayout wird in der Hoffnung, dass es nützlich sein wird, aber
  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
  Siehe die GNU General Public License für weitere Details.

  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
*/

/*
**  © 2011-2012 Carsten Gaede (PiperSoft)
**
**  @project: ZitherLayout
**  @file: output.c
**  @brief implementation of (latex) output module
**
**  @date: 2011-11-13
**  @author: Carsten Gaede
**
**
**  @version 1.0.0.0 2012-01-01 Carsten Gaede
**    first official release
**  @version 0.0.0.1 2011-11-13 Carsten Gaede
**    initial creation
*/

#include "output.h"

#include <string.h>

#include "layout.h"


// dimensions, all in 1/10 mm
#define DIM_UNIT_LENGTH      1  // 0.1mm
#define DIM_TO_PIC_UNIT(x)  ((x) / DIM_UNIT_LENGTH)
#define DIM_TO_MILLI(x)        ((x) / 10)   // to mm
#define DIM_TO_MICRO(x)        ((x) * 100)  // to µm


int ExportNote(FILE *lpFile, unsigned char len)
{
  if (len & LENGTH_PT) {
    fprintf(lpFile, "\\pt{1}");
  }
  switch(len & LENGTH_MASK) {
    case LENGTH_1:
      fprintf(lpFile, "\\wh{1}");
    break;
    case LENGTH_2:
      fprintf(lpFile, "\\hu{1}");
    break;
    case LENGTH_4:
      fprintf(lpFile, "\\qu{1}");
    break;
    case LENGTH_8:
      fprintf(lpFile, "\\cu{1}");
    break;
    case LENGTH_16:
      fprintf(lpFile, "\\ccu{1}");
    break;
    case LENGTH_32:
      fprintf(lpFile, "\\cccu{1}");
    break;
    case LENGTH_64:
      fprintf(lpFile, "\\ccccu{1}");
    break;
  }
  return(0);
}
int ExportPause(FILE *lpFile, unsigned char len)
{
  if (((len & LENGTH_MASK) >= LENGTH_4) && (len & LENGTH_PT)) {
    fprintf(lpFile, "\\pt{1}");
  }
  switch(len & LENGTH_MASK) {
    case LENGTH_1:
      if (len & LENGTH_PT) {
        fprintf(lpFile, "\\liftpausep{0}");
      }
      else {
        fprintf(lpFile, "\\liftpause{0}");
      }
    break;
    case LENGTH_2:
      if (len & LENGTH_PT) {
        fprintf(lpFile, "\\lifthpausep{0}");
      }
      else {
        fprintf(lpFile, "\\lifthpause{0}");
      }
    break;
    case LENGTH_4:
      fprintf(lpFile, "\\qp{1}");
    break;
    case LENGTH_8:
      fprintf(lpFile, "\\ds{1}");
    break;
    case LENGTH_16:
      fprintf(lpFile, "\\qs{1}");
    break;
    case LENGTH_32:
      fprintf(lpFile, "\\hs{1}");
    break;
    case LENGTH_64:
      fprintf(lpFile, "\\qqs{1}");
    break;
  }
  return(0);
}
int ExportSong(FILE *lpFile, struct SETTINGS *set, struct SONG *song)
{
  int iNote;
  struct NOTES *lpNotes = &song->notes;
  struct NOTE *lpNote;
  int x, y, px, py, h, w, r;
  unsigned char b;
  char c[16];

  // print start needle
  if (lpNotes->count > 0) {
    lpNote = &lpNotes->notes[0];
    x = DIM_TO_PIC_UNIT(lpNote->pos.x);
    py =  DIM_TO_PIC_UNIT(song->start_needle_pos.y);
    y = DIM_TO_PIC_UNIT(lpNote->pos.y);
    r = DIM_TO_PIC_UNIT(set->dim.notes.start_needle_size);
    fprintf(lpFile, "    \\color{red}\r\n");
    fprintf(lpFile, "    \\put(%i,%i){\\circle*{%i}}\r\n", x, py, r);
    fprintf(lpFile, "    \\drawline(%i,%i)(%i,%i)\r\n", x, py, x, y);
    if (strlen(song->warning) > 0) {
      x = DIM_TO_PIC_UNIT(lpNote->pos.x + set->dim.notes.chord.dx_l);
      y = DIM_TO_PIC_UNIT(song->start_needle_pos.y + set->dim.notes.chord.dy_lr);
      if (set->compression.current <= MAX_COMPRESSION_NORMAL_SIZE) {
        fprintf(lpFile, "    \\put(%i,%i){\\large{*}}\r\n", x, y);
      }
      else if (set->compression.current <= MAX_COMPRESSION_SMALL_SIZE) {
        fprintf(lpFile, "    \\put(%i,%i){*}\r\n", x, y);
      }
      else {
        fprintf(lpFile, "    \\put(%i,%i){\\small{*}}\r\n", x, y);
      }
    }
    fprintf(lpFile, "    \\color{black}\r\n");
  }
  // print note connecting lines
  for (iNote = 0; iNote < lpNotes->count; ++iNote) {
    lpNote = &lpNotes->notes[iNote];
    x = lpNote->pos.x;
    y = lpNote->pos.y;
    if (iNote > 0) {
      fprintf(lpFile, "    \\drawline(%i,%i)(%i,%i)\r\n", px, py, x, y);
    }
    px = x;
    py = y;
  }
  // erase circle around notes
  fprintf(lpFile, "    \\color{white}\r\n");
  for (iNote = 0; iNote < lpNotes->count; ++iNote) {
    lpNote = &lpNotes->notes[iNote];
    x = DIM_TO_PIC_UNIT(lpNote->pos.x);
    y = DIM_TO_PIC_UNIT(lpNote->pos.y);
    r = DIM_TO_PIC_UNIT(set->dim.notes.circle_size);
    fprintf(lpFile, "    \\put(%i,%i){\\circle*{%i}}\r\n", x, y, r);
  }
  fprintf(lpFile, "    \\color{black}\r\n");
  // print notes
  for (iNote = 0; iNote < lpNotes->count; ++iNote) {
    lpNote = &lpNotes->notes[iNote];
    x = DIM_TO_PIC_UNIT(lpNote->pos.x + set->dim.notes.dx);
    y = DIM_TO_PIC_UNIT(lpNote->pos.y + set->dim.notes.dy);
    fprintf(lpFile, "    \\put(%i,%i){", x, y);
    if (lpNote->value == VALUE_PAUSE) {
      ExportPause(lpFile, lpNote->length);
    }
    else {
      ExportNote(lpFile, lpNote->length);
    }
    fprintf(lpFile, "}\r\n");
  }
  // print chords
  for (iNote = 0; iNote < lpNotes->count; ++iNote) {
    lpNote = &lpNotes->notes[iNote];
    x = lpNote->pos.x;
    y = lpNote->pos.y;
    if (lpNote->chord != 0) {
      switch(lpNote->chord_pos) {
        case RPOS_LEFT:
          x += set->dim.notes.chord.dx_l;
          y += set->dim.notes.chord.dy_lr;
        break;
        case RPOS_RIGHT:
          x += set->dim.notes.chord.dx_r;
          y += set->dim.notes.chord.dy_lr;
        break;
        case RPOS_BOTTOM:
          x += set->dim.notes.chord.dx_tb;
          y += set->dim.notes.chord.dy_b;
        break;
      }
      x = DIM_TO_PIC_UNIT(x);
      y = DIM_TO_PIC_UNIT(y);
      fprintf(lpFile, "    \\put(%i,%i){", x, y);
      if (lpNote->chord & CHORD_SLR) {
        fprintf(lpFile, "\\ul{");
      }
      if (set->compression.current <= MAX_COMPRESSION_NORMAL_SIZE) {
        fprintf(lpFile, "\\large{%i}", lpNote->chord & CHORD_VALUE_MASK);
      }
      else if (set->compression.current <= MAX_COMPRESSION_SMALL_SIZE) {
        fprintf(lpFile, "%i", lpNote->chord & CHORD_VALUE_MASK);
      }
      else {
        fprintf(lpFile, "\\small{%i}", lpNote->chord & CHORD_VALUE_MASK);
      }
      if (lpNote->chord & CHORD_SLR) {
        fprintf(lpFile, "}");
      }
      fprintf(lpFile, "}\r\n");
    }
  }

  // print my name... :)
  y = DIM_TO_PIC_UNIT(set->dim.picture.height - set->dim.picture.notes.borders.top);
  fprintf(lpFile, "    \\put(0,%i){\\makebox(0,0)[tl]{\\rotatebox{90}{\\small{%s: Carsten Gaede}}}}\r\n", y, TEXT_LAYOUT);
  // print title and other texts
  x = DIM_TO_PIC_UNIT(set->dim.picture.info.width / 2);
  y = DIM_TO_PIC_UNIT(set->dim.picture.height - set->dim.picture.notes.borders.top);
  w = DIM_TO_MILLI(set->dim.picture.info.width);
  fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[t]{\\parbox[t]{%imm}{\\centering{\\textbf{\\textit{\\LARGE{%s $3 \\tfrac{1}{2}$}}}}}}}\r\n", x, y, w, TEXT_ZITHER1);
  y -= DIM_TO_PIC_UNIT(70);
  fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[t]{\\parbox[t]{%imm}{\\centering{\\textbf{\\textit{(%i %s)}}}}}}\r\n", x, y, w, set->count.chords, TEXT_ZITHER2);
  b = 0;
  x = set->count.rows / 2;
  for(iNote = 0; iNote < lpNotes->count; ++iNote) {
    if ((lpNotes->notes[iNote].pos.row > x) && (lpNotes->notes[iNote].value == 1)) {
      b = 1;
      if (lpNotes->notes[iNote].chord != 0) {
        b = 2;
        break;
      }
    }
  }
  switch(b) {
    case 0:
      sprintf(c, "bl");
    break;
    case 1:
      sprintf(c, "b");
    break;
    case 2:
      sprintf(c, "br");
    break;
  }
  x = DIM_TO_PIC_UNIT(set->dim.picture.notes.offset);
  y = DIM_TO_PIC_UNIT(set->dim.picture.notes.borders.bottom);
  fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[%s]{\\rotatebox{90}{\\large{\\textit{%s}}}}}\r\n", x, y, c, TEXT_C1_LINE);
  if (strlen(song->title) > 0) {
    if (strlen(song->subtitle) > 0) {
      sprintf(c, "r");
    }
    else {
      sprintf(c, "c");
    }
    x = DIM_TO_PIC_UNIT(set->dim.picture.info.width / 2);
    y = DIM_TO_PIC_UNIT(set->dim.picture.height / 2);
    fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[%s]{\\rotatebox{90}{\\Huge{\\textbf{%s}}}}}\r\n", x, y, c, song->title);
    if (strlen(song->subtitle) > 0) {
      x += DIM_TO_PIC_UNIT(set->dim.picture.info.title.sub_title_dist);
      w = DIM_TO_MILLI(set->dim.picture.info.title.width);
      fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[l]{\\rotatebox{90}{\\parbox{%imm}{\\centering{\\Large{%s}}}}}}\r\n", x, y, w, song->subtitle);
    }
  }
  if ((strlen(song->artists.composer) > 0) || (strlen(song->artists.texter) > 0) || (strlen(song->artists.arrangement) > 0)) {
    x = DIM_TO_PIC_UNIT(set->dim.picture.info.width);
    y = DIM_TO_PIC_UNIT(set->dim.picture.height - set->dim.picture.notes.borders.top);
    w = DIM_TO_MILLI((set->dim.picture.height / 2) - set->dim.picture.notes.borders.top);
    fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[tr]{\\rotatebox{90}{\\parbox{%imm}{\\raggedleft{\\large{", x, y, w);
    if ((strlen(song->artists.composer) > 0) || (strlen(song->artists.texter) > 0)) {
      if (strcmp(song->artists.composer, song->artists.texter) == 0) {
        fprintf(lpFile, "%s: \\textbf{%s}", TEXT_ARTIST, song->artists.composer);
      }
      else {
        if (strlen(song->artists.composer) > 0) {
          fprintf(lpFile, "%s: \\textbf{%s}", TEXT_COMPOSER, song->artists.composer);
        }
        if ((strlen(song->artists.composer) > 0) && (strlen(song->artists.texter) > 0)) {
          fprintf(lpFile, "\\\\ ");
        }
        if (strlen(song->artists.texter) > 0) {
          fprintf(lpFile, "%s: \\textbf{%s}", TEXT_TEXTER, song->artists.texter);
        }
      }
    }
    if (((strlen(song->artists.composer) > 0) || (strlen(song->artists.texter) > 0)) && (strlen(song->artists.arrangement) > 0)) {
      fprintf(lpFile, "\\\\ ");
    }
    if (strlen(song->artists.arrangement) > 0) {
      fprintf(lpFile, "%s: \\textbf{%s}", TEXT_ARRANGEMENT, song->artists.arrangement);
    }
    fprintf(lpFile, "}}}}}}\r\n");
  }
  if ((strlen(song->warning) > 0) || (strlen(song->hint) > 0)) {
    x = 0;
    y = DIM_TO_PIC_UNIT(set->dim.picture.notes.borders.bottom);
    h = DIM_TO_MILLI(set->dim.picture.height - set->dim.picture.notes.borders.top - set->dim.picture.notes.borders.bottom);
    fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[bl]{\\rotatebox{90}{\\parbox[t]{%imm}{", x, y, h);
    if (strlen(song->warning) > 0) {
      fprintf(lpFile, "\\color{red}*%s: %s\\color{black}", TEXT_WARNING, song->warning);
    }
    if ((strlen(song->warning) > 0) && (strlen(song->hint) > 0)) {
      fprintf(lpFile, "\\\\ ");
    }
    if (strlen(song->hint) > 0) {
      fprintf(lpFile, "%s: %s", TEXT_HINT, song->hint);
    }
    fprintf(lpFile, "}}}}\r\n");
  }

  // print lyrics
  for (iNote = 0; iNote < song->lyrics.count; ++iNote) {
    if (strlen(song->lyrics.lyrics[iNote].text)) {
      x = DIM_TO_PIC_UNIT(song->lyrics.lyrics[iNote].pos.x);
      y = DIM_TO_PIC_UNIT(song->lyrics.lyrics[iNote].pos.y);
      w = DIM_TO_MILLI(set->dim.picture.width - song->lyrics.lyrics[iNote].pos.x);
      fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[tl]{\\parbox{%imm}{%s}}}\r\n", x, y, w, song->lyrics.lyrics[iNote].text);
      if (set->lyric_numbers) {
        x = DIM_TO_PIC_UNIT(song->lyrics.lyrics[iNote].pos.x - set->dim.lyrics.number_dist);
        fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[tr]{%i.}}\r\n", x, y, iNote + 1);
      }
    }
  }

  return(0);
}

#define PrintDim_mm(str, dim) \
  sprintf(str, "%i.%imm", DIM_TO_MILLI(dim), DIM_TO_MICRO(dim) % 1000)
int ExportLaTeX(FILE *lpFile, struct SESSION *s)
{
  char pbt[16],pbb[16],pbl[16],pbr[16];
  int i;
  int x, y, x1, y1, x2, y2;
  int h, w;

  fprintf(lpFile, "\\documentclass[a4paper,10pt,landscape]{scrartcl}\r\n");
  fprintf(lpFile, "\\usepackage[paper=a4paper,landscape,tmargin=0pt,lmargin=0pt]{geometry}\r\n");
  fprintf(lpFile, "\\usepackage[utf8x]{inputenc}\r\n");
  fprintf(lpFile, "\\usepackage{epic,eepic}\r\n");
  fprintf(lpFile, "\\usepackage{color}\r\n");
  fprintf(lpFile, "\\usepackage{graphpap}\r\n");
  fprintf(lpFile, "\\usepackage{soul}\r\n");
  fprintf(lpFile, "\\usepackage{zitherlayout}\r\n");
  fprintf(lpFile, "\\usepackage{vmargin}\r\n");
  fprintf(lpFile, "\\usepackage{rotating}\r\n");
  fprintf(lpFile, "\\usepackage{amsmath}\r\n");
  fprintf(lpFile, "\r\n");
  fprintf(lpFile, "\\setpapersize[landscape]{A4}\r\n");
  PrintDim_mm(pbt, s->set.dim.page.borders.top);
  PrintDim_mm(pbb, s->set.dim.page.borders.bottom);
  PrintDim_mm(pbl, s->set.dim.page.borders.left);
  PrintDim_mm(pbr, s->set.dim.page.borders.right);
  fprintf(lpFile, "\\setmarginsrb{%s}{%s}{%s}{%s}{0pt}{0pt}{0pt}{0pt}\r\n", pbl, pbt, pbr, pbb);
  fprintf(lpFile, "\\begin{document}\r\n");
  fprintf(lpFile, "  \\pagestyle{empty}\r\n");
  PrintDim_mm(pbt, DIM_UNIT_LENGTH);
  fprintf(lpFile, "  \\setlength{\\unitlength}{%s}\r\n", pbt);
  fprintf(lpFile, "  \\setuldepth{");
  for (i = 1; i <= s->set.count.chords; ++i) {
    fprintf(lpFile, "%i", i);
  }
  fprintf(lpFile, "}\r\n");
  if (s->set.compression.current <= MAX_COMPRESSION_NORMAL_SIZE) {
    fprintf(lpFile, "  \\normalnotesize\r\n");
  }
  else if (s->set.compression.current <= MAX_COMPRESSION_SMALL_SIZE) {
    fprintf(lpFile, "  \\smallnotesize\r\n");
  }
  else {
    fprintf(lpFile, "  \\tinynotesize\r\n");
  }
  fprintf(lpFile, "  \\begin{picture}(%i,%i)\r\n", s->set.dim.picture.width, s->set.dim.picture.height);
  fprintf(lpFile, "    \\color{black}\r\n");

  // print C line
  fprintf(lpFile, "    \\thicklines\r\n");
  x = s->set.dim.picture.notes.offset + s->set.dim.picture.notes.borders.left;
  fprintf(lpFile, "    \\drawline(%i,%i)(%i,%i)\r\n", x, 0, x, s->set.dim.picture.height);
  fprintf(lpFile, "    \\thinlines\r\n");

  ExportSong(lpFile, &s->set, &s->song);

  if (s->set.grid) {
    // print picture border and grid
    fprintf(lpFile, "    \\color{green}\r\n");
    h = DIM_TO_PIC_UNIT(s->set.dim.picture.height);
    w = DIM_TO_PIC_UNIT(s->set.dim.picture.width);
    fprintf(lpFile, "    \\drawline(0,0)(0,%i)(%i,%i)(%i,0)(0,0)\r\n", h, w, h, w);
    x = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset);
    fprintf(lpFile, "    \\drawline(%i,0)(%i,%i)\r\n", x, x, h);
    fprintf(lpFile, "    \\color{blue}\r\n");
    y = h;
    for (i = 0; i < s->set.count.cols; i += 5) {
      x = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset + s->set.dim.picture.notes.borders.left + (i * s->set.dim.picture.col.width));
      fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[t]{%i}}\r\n", x, h, i);
    }
    x = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset);
    for (i = 0; i < s->set.count.rows; i += 5) {
      y = DIM_TO_PIC_UNIT(s->set.dim.picture.height - s->set.dim.picture.notes.borders.top - (i * s->set.dim.picture.row.height));
      fprintf(lpFile, "    \\put(%i,%i){\\makebox(0,0)[l]{%i}}\r\n", s->set.dim.picture.notes.offset, y, i);
    }
    x1 = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset + s->set.dim.picture.notes.borders.left);
    x2 = DIM_TO_PIC_UNIT(s->set.dim.picture.col.width);
    h = DIM_TO_PIC_UNIT(s->set.dim.picture.height);
    fprintf(lpFile, "    \\matrixput(%i,0)(%i,0){%i}(0,0){1}{\\line(0,1){%i}}\r\n", x1, x2, s->set.count.cols, h);
    x = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset);
    y1 = DIM_TO_PIC_UNIT(s->set.dim.picture.height - s->set.dim.picture.notes.borders.top);
    y2 = -DIM_TO_PIC_UNIT(s->set.dim.picture.row.height);
    w = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.width);
    fprintf(lpFile, "    \\matrixput(%i,%i)(0,%i){%i}(0,0){1}{\\line(1,0){%i}}\r\n", x, y1, y2, s->set.count.rows, w);
    fprintf(lpFile, "    \\thicklines\r\n");
    x1 = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.offset + s->set.dim.picture.notes.borders.left);
    x2 = DIM_TO_PIC_UNIT(s->set.dim.picture.width - s->set.dim.picture.notes.borders.right);
    y1 = DIM_TO_PIC_UNIT(s->set.dim.picture.notes.borders.bottom);
    y2 = DIM_TO_PIC_UNIT(s->set.dim.picture.height - s->set.dim.picture.notes.borders.top);
    fprintf(lpFile, "    \\drawline(%i,%i)(%i,%i)(%i,%i)(%i,%i)(%i,%i)\r\n", x1, y1, x1, y2, x2, y2, x2, y1, x1, y1);
    fprintf(lpFile, "    \\color{red}\r\n");
    x1 = DIM_TO_PIC_UNIT(s->set.dim.picture.width - s->set.dim.cut.width);
    y1 = DIM_TO_PIC_UNIT(s->set.dim.picture.height);
    x2 = DIM_TO_PIC_UNIT(s->set.dim.picture.width);
    y2 = DIM_TO_PIC_UNIT(s->set.dim.picture.height - s->set.dim.cut.height);
    fprintf(lpFile, "    \\drawline(%i,%i)(%i,%i)\r\n", x1, y1, x2, y2);
    fprintf(lpFile, "    \\thinlines\r\n");
    fprintf(lpFile, "    \\color{black}\r\n");
  }

  fprintf(lpFile, "  \\end{picture}\r\n");
  fprintf(lpFile, "\\end{document}\r\n");

  return(0);
}
